@@ -68,7 +68,7 @@ static PyObject *_mysql_IntegrityError;
 static PyObject *_mysql_InternalError; 
 static PyObject *_mysql_ProgrammingError;
 static PyObject *_mysql_NotSupportedError;
- 
+
 typedef struct {
 	PyObject_HEAD
 	MYSQL connection;
@@ -88,6 +88,7 @@ typedef struct {
 	MYSQL_RES *result;
 	int nfields;
 	int use;
+	char has_next;
 	PyObject *converter;
 } _mysql_ResultObject;
 
@@ -193,6 +194,9 @@ _mysql_Exception(_mysql_ConnectionObject *c)
 #endif
 #ifdef ER_CANNOT_ADD_FOREIGN
 	case ER_CANNOT_ADD_FOREIGN:
+#endif
+#ifdef ER_NO_DEFAULT_FOR_FIELD
+	case ER_NO_DEFAULT_FOR_FIELD:
 #endif
 		e = _mysql_IntegrityError;
 		break;
@@ -223,7 +227,7 @@ _mysql_Exception(_mysql_ConnectionObject *c)
 	Py_DECREF(t);
 	return NULL;
 }
-	  
+
 static char _mysql_server_init__doc__[] =
 "Initialize embedded server. If this client is not linked against\n\
 the embedded server library, this function does nothing.\n\
@@ -246,7 +250,7 @@ static PyObject *_mysql_server_init(
 				"already initialized");
 		return NULL;
 	}
-	  
+
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|OO", kwlist,
 					 &cmd_args, &groups))
 		return NULL;
@@ -345,7 +349,7 @@ static PyObject *_mysql_server_end(
 	}
 	return _mysql_Exception(NULL);
 }
-	 
+
 #if MYSQL_VERSION_ID >= 32314
 static char _mysql_thread_safe__doc__[] =
 "Indicates whether the client is compiled as thread-safe.";
@@ -403,9 +407,10 @@ _mysql_ResultObject_Initialize(
 	else
 		result = mysql_store_result(&(conn->connection));
 	self->result = result;
+	self->has_next = (char)mysql_more_results(&(conn->connection));
 	Py_END_ALLOW_THREADS ;
 	if (!result) {
-		if (mysql_field_count(&(conn->connection)) > 0) {
+		if (mysql_errno(&(conn->connection))) {
 		    _mysql_Exception(conn);
 		    return -1;
 		}
@@ -552,7 +557,7 @@ _mysql_ConnectionObject_Initialize(
 	char *init_command=NULL,
 	     *read_default_file=NULL,
 	     *read_default_group=NULL;
-	
+
 	self->converter = NULL;
 	self->open = 0;
 	check_server_init(-1);
@@ -736,7 +741,7 @@ _mysql_connect(
 	PyObject *kwargs)
 {
 	_mysql_ConnectionObject *c=NULL;
-	
+
 	c = MyAlloc(_mysql_ConnectionObject, _mysql_ConnectionObject_Type);
 	if (c == NULL) return NULL;
 	if (_mysql_ConnectionObject_Initialize(c, args, kwargs)) {
@@ -1079,7 +1084,7 @@ _mysql_escape_string(
 	PyObject *str;
 	char *in, *out;
 	int len, size;
-	if (!PyArg_ParseTuple(args, "y#:escape_string", &in, &size)) return NULL;
+	if (!PyArg_ParseTuple(args, "s#:escape_string", &in, &size)) return NULL;
 	str = PyBytes_FromStringAndSize((char *) NULL, size*2+1);
 	if (!str) return PyErr_NoMemory();
 	out = PyBytes_AS_STRING(str);
@@ -1286,7 +1291,7 @@ _mysql_escape_dict(
 	Py_XDECREF(r);
 	return NULL;
 }
-				
+
 static char _mysql_ResultObject_describe__doc__[] =
 "Returns the sequence of 7-tuples required by the DB-API for\n\
 the Cursor.description attribute.\n\
@@ -1323,7 +1328,7 @@ _mysql_ResultObject_describe(
 	Py_XDECREF(d);
 	return NULL;
 }
-	
+
 static char _mysql_ResultObject_field_flags__doc__[] =
 "Returns a tuple of field flags, one for each column in the result.\n\
 " ;
@@ -1369,7 +1374,9 @@ _mysql_field_to_python(
 		field_type == FIELD_TYPE_LONG_BLOB ||
 		field_type == FIELD_TYPE_BLOB ||
 		field_type == FIELD_TYPE_VAR_STRING ||
-		field_type == FIELD_TYPE_STRING) {
+		field_type == FIELD_TYPE_STRING ||
+		field_type == FIELD_TYPE_GEOMETRY ||
+		field_type == FIELD_TYPE_BIT) {
 			binary = 1;
 	}
 #endif
@@ -1515,7 +1522,7 @@ _mysql__fetch_row(
 	int maxrows,
 	_PYFUNC *convert_row)
 {
-	unsigned int i;
+	int i;
 	MYSQL_ROW row;
 
 	for (i = skiprows; i<(skiprows+maxrows); i++) {
@@ -1568,14 +1575,14 @@ _mysql_ResultObject_fetch_row(
 		_mysql_row_to_dict_old
 	};
 	_PYFUNC *convert_row;
-	unsigned int maxrows=1, how=0, skiprows=0, rowsadded;
+	int maxrows=1, how=0, skiprows=0, rowsadded;
 	PyObject *r=NULL;
 
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "|ii:fetch_row", kwlist,
 					 &maxrows, &how))
 		return NULL;
 	check_result_connection(self);
-	if (how >= sizeof(row_converters)) {
+	if (how >= (int)sizeof(row_converters)) {
 		PyErr_SetString(PyExc_ValueError, "how out of range");
 		return NULL;
 	}
@@ -1729,7 +1736,7 @@ _mysql_ConnectionObject_get_character_set_info(
 {
 	PyObject *result;
 	MY_CHARSET_INFO cs;
-	
+
 	if (!PyArg_ParseTuple(args, "")) return NULL;
 	check_connection(self);
 	mysql_get_character_set_info(&(self->connection), &cs);
@@ -2648,6 +2655,13 @@ static struct PyMemberDef _mysql_ResultObject_memberlist[] = {
 		READONLY,
 		"Type conversion mapping"
 	},
+	{
+		"has_next",
+		T_BOOL,
+		offsetof(_mysql_ResultObject, has_next),
+		READONLY,
+		"Has next result"
+	},
 	{NULL} /* Sentinel */
 };
 
@@ -2712,44 +2726,44 @@ PyTypeObject _mysql_ConnectionObject_Type = {
 	0, /* tp_setattr */
 	0, /*tp_compare*/
 	(reprfunc)_mysql_ConnectionObject_repr, /* tp_repr */
-	
+
 	/* Method suites for standard classes */
-	
+
 	0, /* (PyNumberMethods *) tp_as_number */
 	0, /* (PySequenceMethods *) tp_as_sequence */
 	0, /* (PyMappingMethods *) tp_as_mapping */
-	
+
 	/* More standard operations (here for binary compatibility) */
-	
+
 	0, /* (hashfunc) tp_hash */
 	0, /* (ternaryfunc) tp_call */
 	0, /* (reprfunc) tp_str */
 	(getattrofunc)_mysql_ConnectionObject_getattro, /* tp_getattro */
 	(setattrofunc)_mysql_ConnectionObject_setattro, /* tp_setattro */
-	
+
 	/* Functions to access object as input/output buffer */
 	0, /* (PyBufferProcs *) tp_as_buffer */
-	
+
 	/* (tp_flags) Flags to define presence of optional/expanded features */
 	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
 	_mysql_connect__doc__, /* (char *) tp_doc Documentation string */
 
 	/* call function for all accessible objects */
 	(traverseproc) _mysql_ConnectionObject_traverse, /* tp_traverse */
-	
+
 	/* delete references to contained objects */
 	(inquiry) _mysql_ConnectionObject_clear, /* tp_clear */
 
 	/* rich comparisons */
 	0, /* (richcmpfunc) tp_richcompare */
-	
+
 	/* weak reference enabler */
 	0, /* (long) tp_weaklistoffset */
 
 	/* Iterators */
 	0, /* (getiterfunc) tp_iter */
 	0, /* (iternextfunc) tp_iternext */
-	
+
 	/* Attribute descriptor and subclassing stuff */
 	(struct PyMethodDef *)_mysql_ConnectionObject_methods, /* tp_methods */
 	(struct PyMemberDef *)_mysql_ConnectionObject_memberlist, /* tp_members */
@@ -2784,45 +2798,45 @@ PyTypeObject _mysql_ResultObject_Type = {
 	0, /* tp_setattr */
 	0, /*tp_compare*/
 	(reprfunc)_mysql_ResultObject_repr, /* tp_repr */
-	
+
 	/* Method suites for standard classes */
-	
+
 	0, /* (PyNumberMethods *) tp_as_number */
 	0, /* (PySequenceMethods *) tp_as_sequence */
 	0, /* (PyMappingMethods *) tp_as_mapping */
-	
+
 	/* More standard operations (here for binary compatibility) */
-	
+
 	0, /* (hashfunc) tp_hash */
 	0, /* (ternaryfunc) tp_call */
 	0, /* (reprfunc) tp_str */
 	(getattrofunc)PyObject_GenericGetAttr, /* tp_getattro */
 	(setattrofunc)_mysql_ResultObject_setattro, /* tp_setattr */
-	
+
 	/* Functions to access object as input/output buffer */
 	0, /* (PyBufferProcs *) tp_as_buffer */
-	
+
 	/* Flags to define presence of optional/expanded features */
 	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
-	
+
 	_mysql_ResultObject__doc__, /* (char *) tp_doc Documentation string */
 
 	/* call function for all accessible objects */
 	(traverseproc) _mysql_ResultObject_traverse, /* tp_traverse */
-	
+
 	/* delete references to contained objects */
 	(inquiry) _mysql_ResultObject_clear, /* tp_clear */
 
 	/* rich comparisons */
 	0, /* (richcmpfunc) tp_richcompare */
-	
+
 	/* weak reference enabler */
 	0, /* (long) tp_weaklistoffset */
 
 	/* Iterators */
 	0, /* (getiterfunc) tp_iter */
 	0, /* (iternextfunc) tp_iternext */
-	
+
 	/* Attribute descriptor and subclassing stuff */
 	(struct PyMethodDef *) _mysql_ResultObject_methods, /* tp_methods */
 	(struct PyMemberDef *) _mysql_ResultObject_memberlist, /*tp_members */
